/*
    This program demonstrates how a "race condition" can come up
    when two threads both manipulate a shared variable. In this
    example, the shared variable is the global variable count.
    A race condition is where the result of a piece of code
    depends on the order in which the processor executes the
    threads.
*/
#include <stdio.h>    // printf()
#include <unistd.h>   // sleep()
#include <pthread.h>
#include <windows.h>    // win32

// global variables
int runFlag = 1;
unsigned int count  = 0;  // this variable is shared by thread1 and thread2
unsigned int count1 = 0;  // this variable is used only by thread1
unsigned int count2 = 0;  // this variable is used only by thread2

int main()
{
   void *thread1(void *vargp);  //thread function prototypes
   void *thread2(void *vargp);

   int threadParam1 = 0;  // thread function parameters
   int threadParam2 = 0;

   pthread_t tid1;
   pthread_t tid2;

   // create two threads
   printf("race1: Starting threads\n");
   pthread_create(&tid1, NULL, thread1, &threadParam1);
   pthread_create(&tid2, NULL, thread2, &threadParam2);

   Sleep(2000);  //sleep(2);     // let the threads run for a while
   runFlag = 0;                  // now stop the threads
   Sleep(1000);  //sleep(1);     // give the threads time to stop

   printf("\nThreads stopped.\n");
   printf("count = %d, count1 = %d, count2 = %d\n", count, count1, count2);

   return 0;
}//main

void *thread1(void *vargp)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      // create a race situation with thread2
      // increment a global variable and store the results locally
      old_count = count++;
      new_count = count;
      // check that the local and global variables were consistent with each other
      if (new_count - old_count != 1)
         printf("Bingo1: new_count - old_count = %d\n", new_count-old_count);
      count1++;
   }
   return NULL;
}//thread1


void *thread2(void *vargp)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      // create a race situation with thread1
      old_count = count++;
      new_count = count;
      // check for the race condition
      if (new_count - old_count != 1)
         printf("Bingo2: new_count - old_count = %d\n", new_count-old_count);
      count2++;
   }
   return NULL;
}//thread2
