/*
    This program demonstrates a very subtle race condition.
    If you compile this program in the usual way then this
    program will NOT have a race condition.

    But if you complile this program using gcc and the compiler
    option -fforce-mem, then you do get a race condition.

    The compiler option -fforce-mem tells gcc to compile in a
    way that mimics RISC type machines. So the assembly language
    code for count++ changes from the usual i386 code with an
    atomic memory access
      incl  _count
    to the more RISC like
      movl  _count, %eax
      incl  %eax
      movl  %eax, _count
    The extra instructions allow for a race condition between the threads.

    This program shows that code can have a race condition that only shows
    up when the code is compiled on certain kinds of CPU's.

    This example is based on Figure 13.16 badcnt.c, page 872,  from
    "Computer Systems: A Programmers Perspective", by R Bryant and D. O'Hallaron.
*/
#include <stdio.h>    // printf()
#include <pthread.h>

#define ITERATIONS 100000000

// global shared variable
unsigned count = 0;

int main( int argc, char *argv[])
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int threadParam = 0;  // thread function parameter
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_create(&(tid[i]), NULL, thread, &threadParam);

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   if (count != (unsigned)ITERATIONS*NUMBER_OF_THREADS)
      printf("A RACE! count=%d (not %d)\n", count, (unsigned)ITERATIONS*NUMBER_OF_THREADS);
   else
      printf("No race: count=%d\n", count);

   return 0;
}//main



void *thread(void *vargp)
{
   int i;
   for (i = 0; i < ITERATIONS; i++)
      count++;

   return NULL;
}//thread
