/*
    This program combines ideas from race-example.c and
    race-example-fixed1.c so that in this program each
    thread has one shared variable that has a race condition
    and each thread has a "private" variable with no race condition.
*/
#include <stdio.h>    // printf()
#include <stdlib.h>   // malloc(), free(), srand(), rand()
#include <pthread.h>


struct vars { int var; int* ptr_var; };

// Structure to hold two thread parameters
typedef struct
{
   int  parameter;
   int* parameter_p;
}ThreadStuff_t;


int main( int argc, char *argv[])
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int i;

   ThreadStuff_t* ptr;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      ptr = (ThreadStuff_t*)malloc(sizeof(ThreadStuff_t));
      (ptr -> parameter)   = i;  // make a copy of i
      (ptr -> parameter_p) = &i; // i is a shared variable
      pthread_create(&(tid[i]), NULL, thread, ptr);
   }

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread(void *threadParam)
{
   int myParameter, sharedParameter;

   // Let the scheduler reschedule these threads in a different order.
   //static int x = 0; // count the number of times this thread is instantiated
   //srand(x++);       // use that to give each thread a different seed value
   //if (rand()%2==0) sleep(0);  // about half the threads will sleep

   myParameter = (*((ThreadStuff_t*)threadParam)).parameter;
   sharedParameter = *((*((ThreadStuff_t*)threadParam)).parameter_p);

   // alternative notation for accessing the thread parameters
   myParameter = ((ThreadStuff_t*)threadParam) -> parameter;
   sharedParameter = *(((ThreadStuff_t*)threadParam) -> parameter_p);

   free(threadParam);  // we no longer need the heap storage space

   fprintf(stderr, "Hello with myParameter = %d and sharedParameter = %d\n", myParameter, sharedParameter);

   return NULL;
}//thread
